/*
 * SPDX-FileCopyrightText: 2023 jordi fita mas <jfita@peritasoft.com>
 * SPDX-License-Identifier: AGPL-3.0-only
 */

package types

import (
	"context"
	"fmt"
	"net/http"
	"strconv"

	"github.com/jackc/pgx/v4"

	"dev.tandem.ws/tandem/camper/pkg/auth"
	"dev.tandem.ws/tandem/camper/pkg/database"
	"dev.tandem.ws/tandem/camper/pkg/form"
	httplib "dev.tandem.ws/tandem/camper/pkg/http"
	"dev.tandem.ws/tandem/camper/pkg/locale"
	"dev.tandem.ws/tandem/camper/pkg/template"
)

func (h *AdminHandler) optionsHandler(user *auth.User, company *auth.Company, conn *database.Conn, typeSlug string) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		var head string
		head, r.URL.Path = httplib.ShiftPath(r.URL.Path)

		switch head {
		case "":
			switch r.Method {
			case http.MethodGet:
				serveOptionIndex(w, r, user, company, conn, typeSlug)
			case http.MethodPost:
				addOption(w, r, user, company, conn, typeSlug)
			default:
				httplib.MethodNotAllowed(w, r, http.MethodGet, http.MethodPost)
			}
		case "new":
			switch r.Method {
			case http.MethodGet:
				f, err := newOptionForm(r.Context(), company, conn, typeSlug)
				if err != nil {
					panic(err)
				}
				f.MustRender(w, r, user, company)
			default:
				httplib.MethodNotAllowed(w, r, http.MethodGet)
			}
		default:
			id, err := strconv.Atoi(head)
			if err != nil {
				http.NotFound(w, r)
				return
			}
			f, err := newOptionForm(r.Context(), company, conn, typeSlug)
			if err != nil {
				panic(err)
			}
			if err := f.FillFromDatabase(r.Context(), conn, id); err != nil {
				if database.ErrorIsNotFound(err) {
					http.NotFound(w, r)
					return
				}
				panic(err)
			}
			h.optionHandler(user, company, conn, f).ServeHTTP(w, r)
		}
	})
}

func (h *AdminHandler) optionHandler(user *auth.User, company *auth.Company, conn *database.Conn, f *optionForm) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		var head string
		head, r.URL.Path = httplib.ShiftPath(r.URL.Path)

		switch head {
		case "":
			switch r.Method {
			case http.MethodGet:
				f.MustRender(w, r, user, company)
			case http.MethodPut:
				editOption(w, r, user, company, conn, f)
			default:
				httplib.MethodNotAllowed(w, r, http.MethodGet, http.MethodPut)
			}
		default:
			loc, ok := h.locales.Get(head)
			if !ok {
				http.NotFound(w, r)
				return
			}
			l10n := newOptionL10nForm(f, loc)
			if err := l10n.FillFromDatabase(r.Context(), conn); err != nil {
				panic(err)
			}
			switch r.Method {
			case http.MethodGet:
				l10n.MustRender(w, r, user, company)
			case http.MethodPut:
				editOptionL10n(w, r, user, company, conn, l10n)
			default:
				httplib.MethodNotAllowed(w, r, http.MethodGet, http.MethodPut)
			}
		}
	})
}

func serveOptionIndex(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn, typeSlug string) {
	options, err := collectOptionEntries(r.Context(), conn, typeSlug)
	if err != nil {
		panic(err)
	}
	page := &optionIndex{
		TypeSlug: typeSlug,
		Options:  options,
	}
	page.MustRender(w, r, user, company)
}

func collectOptionEntries(ctx context.Context, conn *database.Conn, typeSlug string) ([]*optionEntry, error) {
	rows, err := conn.Query(ctx, `
		select '/admin/campsites/types/' || campsite_type.slug || '/options/' || campsite_type_option_id
		     , option.name
		     , array_agg((lang_tag, endonym, not exists (select 1 from campsite_type_option_i18n as i18n where i18n.campsite_type_option_id = option.campsite_type_option_id and i18n.lang_tag = language.lang_tag)) order by endonym)
		from campsite_type_option as option
		    join campsite_type using (campsite_type_id)
		    join company using (company_id)
		   , language
		where lang_tag <> default_lang_tag
		  and language.selectable
		  and campsite_type.slug = $1
		group by campsite_type_option_id
		       , campsite_type.slug
		       , option.name
		order by name
	`, pgx.QueryResultFormats{pgx.BinaryFormatCode}, typeSlug)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	var options []*optionEntry
	for rows.Next() {
		option := &optionEntry{}
		var translations database.RecordArray
		if err = rows.Scan(&option.URL, &option.Name, &translations); err != nil {
			return nil, err
		}
		for _, el := range translations.Elements {
			option.Translations = append(option.Translations, &locale.Translation{
				URL:     option.URL + "/" + el.Fields[0].Get().(string),
				Endonym: el.Fields[1].Get().(string),
				Missing: el.Fields[2].Get().(bool),
			})
		}
		options = append(options, option)
	}

	return options, nil
}

type optionEntry struct {
	URL          string
	Name         string
	Translations []*locale.Translation
}

type optionIndex struct {
	TypeSlug string
	Options  []*optionEntry
}

func (page *optionIndex) MustRender(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company) {
	template.MustRenderAdmin(w, r, user, company, "campsite/option/index.gohtml", page)
}

func addOption(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn, typeSlug string) {
	f, err := newOptionForm(r.Context(), company, conn, typeSlug)
	if err != nil {
		panic(err)
	}
	processOptionForm(w, r, user, company, conn, f, func(ctx context.Context, tx *database.Tx) error {
		id, err := tx.GetInt(ctx, "select add_campsite_type_option($1, $2, $3, $4)", typeSlug, f.Name, f.Min, f.Max)
		if err != nil {
			return err
		}
		return setOptionPrices(ctx, tx, id, f.Prices)
	})
}

func editOption(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn, f *optionForm) {
	processOptionForm(w, r, user, company, conn, f, func(ctx context.Context, tx *database.Tx) error {
		if _, err := conn.Exec(ctx, "select edit_campsite_type_option($1, $2, $3, $4)", f.ID, f.Name, f.Min, f.Max); err != nil {
			return err
		}
		return setOptionPrices(ctx, tx, f.ID, f.Prices)
	})
}

func processOptionForm(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn, f *optionForm, act func(ctx context.Context, tx *database.Tx) error) {
	if ok, err := form.Handle(f, w, r, user); err != nil {
		return
	} else if !ok {
		f.MustRender(w, r, user, company)
		return
	}

	tx := conn.MustBegin(r.Context())
	if err := act(r.Context(), tx); err == nil {
		tx.Commit(r.Context())
	} else {
		tx.Rollback(r.Context())
		panic(err)
	}
	httplib.Redirect(w, r, "/admin/campsites/types/"+f.TypeSlug+"/options", http.StatusSeeOther)
}

func setOptionPrices(ctx context.Context, tx *database.Tx, id int, prices map[int]*optionPriceForm) error {
	for seasonID, p := range prices {
		if _, err := tx.Exec(ctx, "select set_campsite_type_option_cost($1, $2, $3)", id, seasonID, p.PricePerNight); err != nil {
			return err
		}
	}
	return nil
}

type optionForm struct {
	ID       int
	TypeSlug string
	Name     *form.Input
	Min      *form.Input
	Max      *form.Input
	Prices   map[int]*optionPriceForm
}

type optionPriceForm struct {
	SeasonName    string
	PricePerNight *form.Input
}

func newOptionForm(ctx context.Context, company *auth.Company, conn *database.Conn, typeSlug string) (*optionForm, error) {
	f := &optionForm{
		TypeSlug: typeSlug,
		Name: &form.Input{
			Name: "name",
		},
		Min: &form.Input{
			Name: "min",
			Val:  "0",
		},
		Max: &form.Input{
			Name: "max",
			Val:  "1",
		},
	}

	rows, err := conn.Query(ctx, "select season_id, name from season where active and company_id = $1", company.ID)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	f.Prices = make(map[int]*optionPriceForm)
	for rows.Next() {
		var id int
		var name string
		if err = rows.Scan(&id, &name); err != nil {
			return nil, err
		}
		f.Prices[id] = &optionPriceForm{
			SeasonName: name,
			PricePerNight: &form.Input{
				Name: fmt.Sprintf("season.%d.price_per_night", id),
				Val:  "0",
			},
		}
	}

	return f, nil
}

func (f *optionForm) FillFromDatabase(ctx context.Context, conn *database.Conn, id int) error {
	f.ID = id
	row := conn.QueryRow(ctx, `
		select name
		     , lower(range)::text
		     , (upper(range) - 1)::text
		from campsite_type_option
		where campsite_type_option_id = $1
	`, id)
	if err := row.Scan(&f.Name.Val, &f.Min.Val, &f.Max.Val); err != nil {
		return err
	}

	rows, err := conn.Query(ctx, `
		select season_id
		     , to_price(cost_per_night)
		from campsite_type_option
		join campsite_type_option_cost using (campsite_type_option_id)
		where campsite_type_option_id = $1
	`, id)
	if err != nil {
		return err
	}
	defer rows.Close()

	for rows.Next() {
		var seasonID int
		var pricePerNight string
		if err := rows.Scan(&seasonID, &pricePerNight); err != nil {
			return err
		}
		if p, ok := f.Prices[seasonID]; ok {
			p.PricePerNight.Val = pricePerNight
		}
	}
	return rows.Err()
}

func (f *optionForm) Parse(r *http.Request) error {
	if err := r.ParseForm(); err != nil {
		return err
	}
	f.Name.FillValue(r)
	f.Min.FillValue(r)
	f.Max.FillValue(r)
	for _, p := range f.Prices {
		p.PricePerNight.FillValue(r)
	}
	return nil
}

func (f *optionForm) Valid(l *locale.Locale) bool {
	v := form.NewValidator(l)
	if v.CheckRequired(f.Name, l.GettextNoop("Name can not be empty.")) {
		v.CheckMinLength(f.Name, 1, l.GettextNoop("Name must have at least one letter."))
	}
	minValidInt := false
	if v.CheckRequired(f.Min, l.GettextNoop("Minimum can not be empty.")) {
		if v.CheckValidInteger(f.Min, l.GettextNoop("Minimum must be an integer number.")) {
			minValidInt = true
			v.CheckMinInteger(f.Min, 0, l.GettextNoop("Minimum must be zero or greater."))
		}
	}
	if v.CheckRequired(f.Max, l.GettextNoop("Maximum can not be empty.")) {
		if v.CheckValidInteger(f.Min, l.GettextNoop("Maximum must be an integer number.")) && minValidInt {
			min, _ := strconv.Atoi(f.Min.Val)
			v.CheckMinInteger(f.Min, min, l.GettextNoop("Maximum must be equal or greater than minimum."))
		}
	}
	for _, p := range f.Prices {
		if v.CheckRequired(p.PricePerNight, l.GettextNoop("Price per night can not be empty.")) {
			if v.CheckValidDecimal(p.PricePerNight, l.GettextNoop("Price per night must be a decimal number.")) {
				v.CheckMinDecimal(p.PricePerNight, 0.0, l.GettextNoop("Price per night must be zero or greater."))
			}
		}
	}
	return v.AllOK
}

func (f *optionForm) MustRender(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company) {
	template.MustRenderAdmin(w, r, user, company, "campsite/option/form.gohtml", f)
}
