/*
 * SPDX-FileCopyrightText: 2023 jordi fita mas <jfita@peritasoft.com>
 * SPDX-License-Identifier: AGPL-3.0-only
 */

package types

import (
	"context"
	gotemplate "html/template"
	"net/http"

	"github.com/jackc/pgx/v4"

	"dev.tandem.ws/tandem/camper/pkg/auth"
	"dev.tandem.ws/tandem/camper/pkg/database"
	httplib "dev.tandem.ws/tandem/camper/pkg/http"
	"dev.tandem.ws/tandem/camper/pkg/locale"
	"dev.tandem.ws/tandem/camper/pkg/template"
	"dev.tandem.ws/tandem/camper/pkg/uuid"
)

type PublicHandler struct {
}

func (h *PublicHandler) Handler(user *auth.User, company *auth.Company, conn *database.Conn) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		var head string
		head, r.URL.Path = httplib.ShiftPath(r.URL.Path)

		switch r.Method {
		case http.MethodGet:
			if !uuid.Valid(head) {
				http.NotFound(w, r)
				return
			}
			page, err := newPublicPage(r.Context(), conn, user.Locale, head)
			if database.ErrorIsNotFound(err) {
				http.NotFound(w, r)
				return
			} else if err != nil {
				panic(err)
			}
			page.MustRender(w, r, user, company, conn)
		default:
			httplib.MethodNotAllowed(w, r, http.MethodGet)
		}
	})
}

type publicPage struct {
	*template.PublicPage
	Name        string
	Prices      []*typePrice
	Description gotemplate.HTML
}

type typePrice struct {
	SeasonName    string
	SeasonColor   string
	MinNights     int
	PricePerNight string
	Options       []*optionPrice
}

type optionPrice struct {
	OptionName    string
	PricePerNight string
}

func newPublicPage(ctx context.Context, conn *database.Conn, loc *locale.Locale, slug string) (*publicPage, error) {
	page := &publicPage{
		PublicPage: template.NewPublicPage(),
	}
	row := conn.QueryRow(ctx, `
		select coalesce(i18n.name, campsite_type.name) as l10n_name
		     , coalesce(i18n.description, campsite_type.description)::text as l10n_description
		from campsite_type
		    left join campsite_type_i18n as i18n on campsite_type.campsite_type_id = i18n.campsite_type_id and i18n.lang_tag = $1
		where slug = $2
		  and active
	`, loc.Language, slug)
	if err := row.Scan(&page.Name, &page.Description); err != nil {
		return nil, err
	}

	rows, err := conn.Query(ctx, `
		select coalesce(i18n.name, season.name) as l10n_name
		     , to_color(season.color)::text
		     , coalesce(cost.min_nights, 1)
		     , to_price(coalesce(cost.cost_per_night, 0))
		     , array_agg((coalesce(option_i18n.name, option.name), to_price(coalesce(option_cost.cost_per_night, 0)))) filter (where option.campsite_type_option_id is not null)
        from season
		    left join season_i18n as i18n on season.season_id = i18n.season_id and i18n.lang_tag = $1
			left join (
			    campsite_type_cost as cost join campsite_type as type on cost.campsite_type_id = type.campsite_type_id and type.slug = $2
		    ) as cost on cost.season_id = season.season_id
        	left join (
        	    select option.*
        	    from campsite_type_option as option
        	        join campsite_type as type on option.campsite_type_id = type.campsite_type_id and type.slug = $2
   	        ) as option on true
        	left join campsite_type_option_i18n as option_i18n on option_i18n.campsite_type_option_id = option.campsite_type_option_id and option_i18n.lang_tag = $1
        	left join campsite_type_option_cost as option_cost on option_cost.campsite_type_option_id = option.campsite_type_option_id and option_cost.season_id = season.season_id
		where season.active
		group by i18n.name
		       , season.name
		       , season.color
		       , cost.min_nights
		       , cost.cost_per_night
    `, pgx.QueryResultFormats{pgx.BinaryFormatCode}, loc.Language, slug)
	if err != nil {
		return nil, err
	}

	for rows.Next() {
		price := &typePrice{}
		var options database.RecordArray
		if err := rows.Scan(&price.SeasonName, &price.SeasonColor, &price.MinNights, &price.PricePerNight, &options); err != nil {
			return nil, err
		}
		for _, el := range options.Elements {
			price.Options = append(price.Options, &optionPrice{
				OptionName:    el.Fields[0].Get().(string),
				PricePerNight: el.Fields[1].Get().(string),
			})
		}
		page.Prices = append(page.Prices, price)
	}

	return page, nil
}

func (p *publicPage) MustRender(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn) {
	p.Setup(r, user, company, conn)
	template.MustRenderPublic(w, r, user, company, "campsite/type.gohtml", p)
}
